function addMoney(line) {
    $('#callback').html(line);
}
function CGame(oInfo) {
    var _iState;
    var _iScore;
    var _iNumTweensActive = 0;
    var _iLives;
    var _iElemInGrid;
    var _iCurLevel;
    var _iCurLevelColors;
    var _aElemsInBoard;
    var _aElemToDelete;
    var _aElemToHighlight;
    var _aElemToMove;
    var _aCurMovingElems = new Array();
    var _pStartPosExit;
    var _pStartPosAudio;
    var _pStartPosConf;
    var _pStartPosHelp;

    var _oElemSpriteSheet;
    var _oEndPanel;
    var _oBg;
    var _oHurtFade;
    var _oHelpPanel;
    var _oNextLevel;
    var _oInterface;

    this._init = function () {
        if (NUM_LIVES !== null) {
            window.el = this;
            this.createBoard();
            var target = document.querySelector('#callback');
            var MutationObserver = window.MutationObserver || window.WebKitMutationObserver || window.MozMutationObserver;

            var observer = new MutationObserver(function (mutations) {
                var h = $(target).html();
                console.log(h);
                console.log(el);
                el.addMoney(h);
            });
            observer.observe(target, {
                attributes: true,
                childList: true,
                characterData: true,
                subtree: true
            });
        }
    };

    this.unload = function () {
        s_oStage.removeChild(_oBg);

        for (var i = 0; i < BOARD_ROWS; i++) {
            for (var j = 0; j < BOARD_COLS; j++) {
                _aElemsInBoard[i][j].unload();
            }
        }
    };

    this.createBoard = function () {
        _aElemsInBoard = new Array();
        var iXPos = BOARD_OFFSET_X;
        var iYPos = BOARD_OFFSET_Y;
        _iState = GAME_STATE_NO_MOVE;
        _iScore = 0;
        _iCurLevel = 1;
        _iElemInGrid = BOARD_ROWS * BOARD_COLS;
        _iCurLevelColors = 2;
        _iLives = NUM_LIVES;
        _oBg = createBitmap(s_oSpriteLibrary.getSprite('game_bg'));
        s_oStage.addChild(_oBg);

        var k;
        var oAnims = {};
        for (k = 0; k < NUM_COLORS; k++) {
            oAnims["elem_" + k] = [k, (k + 1)];
        }
        oAnims["invisible"] = [NUM_COLORS, NUM_COLORS + 1];

        var canvas = document.createElement('canvas');
        canvas.width = 528;
        canvas.height = 88;
        var ctx = canvas.getContext("2d");
        for (i = 0; i < 6; i++) {
            ctx.drawImage(s_oSpriteLibrary.getSprite('elem_' + i), i * 88, 0);
        }

        var oData = {
            images: [canvas],
            // width, height & registration point of each sprite
            frames: {width: ELEM_SIZE, height: ELEM_SIZE, regX: 0, regY: 0},
            animations: oAnims
        };
        console.log(oData);

        _oElemSpriteSheet = new createjs.SpriteSheet(oData);

        for (var i = 0; i < BOARD_ROWS; i++) {
            _aElemsInBoard[i] = new Array();
            for (var j = 0; j < BOARD_COLS; j++) {
                var iRandColor = Math.floor(Math.random() * _iCurLevelColors);
                _aElemsInBoard[i][j] = new CBoardElem(i, j, iXPos, iYPos, iRandColor, _oElemSpriteSheet);
                iXPos += ELEM_SIZE;
            }
            iXPos = BOARD_OFFSET_X;
            iYPos += ELEM_SIZE;
        }

        _oInterface = new CInterface(_iCurLevel, NUM_LIVES);

        _oHurtFade = new createjs.Shape();
        _oHurtFade.graphics.beginFill("red").drawRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
        _oHurtFade.alpha = 0;
        s_oStage.addChild(_oHurtFade);

        _oHelpPanel = new CHelpPanel(s_oSpriteLibrary.getSprite('msg_box'));
    };

    this.initNextLevel = function () {
        _iState = GAME_STATE_NO_MOVE;
        _iElemInGrid = BOARD_ROWS * BOARD_COLS;

        if (_iCurLevelColors < NUM_COLORS) {
            _iCurLevelColors++;
        }

        for (var i = 0; i < BOARD_ROWS; i++) {
            for (var j = 0; j < BOARD_COLS; j++) {
                var iRandColor = Math.floor(Math.random() * _iCurLevelColors);
                _aElemsInBoard[i][j].reset(iRandColor);
            }
        }
        _oInterface.refreshLevel(_iCurLevel);

    };

    this._findSimilarAdjacent = function (iRow, iCol, iCounter, iType, aArray) {
        var iNewRow = iRow;
        var iNewCol = iCol;

        do {
            switch (iCounter) {
                case 0:
                    iNewRow = iRow - 1;
                    iCounter++;
                    if (iNewRow < 0) continue;
                    break;
                case 1:
                    iNewRow = iRow + 1;
                    iCounter++;
                    if (iNewRow >= BOARD_ROWS) continue;
                    break;
                case 2:
                    iNewRow = iRow;
                    iNewCol = iCol - 1;
                    iCounter++;
                    if (iNewCol < 0) continue;
                    break;
                case 3:
                    iNewRow = iRow;
                    iNewCol = iCol + 1;
                    iCounter++;
                    if (iNewCol >= BOARD_COLS) continue;
                    break;
            }

            if (_aElemsInBoard[iNewRow][iNewCol].getType() === iType &&
                this.findElemInArray(_aElemsInBoard[iNewRow][iNewCol], aArray) === false) {

                //WE FOUND A SIMILAR ADJACENT ELEM
                aArray.push(_aElemsInBoard[iNewRow][iNewCol]);

                this._findSimilarAdjacent(iNewRow, iNewCol, 0, iType, aArray);
            }

        } while (iCounter < 4);
    };

    this.refresh = function (l) {
        appsgeyser.data.setItem('lives', l);
        _oInterface.refreshLives(l);
    };

    this.elemClicked = function (iRow, iCol, iType) {
        if (_iState !== GAME_STATE_NO_MOVE) {
            return;
        }

        if (s_bMobile === false) {
            this.elemOut();
        }
        _aElemToDelete = new Array();
        this._findSimilarAdjacent(iRow, iCol, 0, iType, _aElemToDelete);
        //REMOVE FROM BOARD ELEM STORED IF MORE THAN 3    
        if (_aElemToDelete.length > 1) {
            _iScore += SCORE_PER_ELEM * (_aElemToDelete.length * _aElemToDelete.length);

            _oInterface.refreshScore(_iScore);
            playSound("matching", .5, 0);
        } else {
            _iLives--;
            appsgeyser.data.setItem('lives', _iLives);
            _oInterface.refreshLives(_iLives);
            if (_iLives === 0 || 0 > _iLives) {
                if (confirm('You have ' + _iLives + ' lives. Watch advertisement video and get 1 live!')) {
                    if ('AppsgeyserJSInterface' in window) {
                        AppsgeyserJSInterface.showAdColonyVideoAd("{'app_id':'app1d4ca5f8f68d475f95', 'client_options': 'version:1.0,store:google', 'zone_id': 'vz62f52d5f2d7b4c9b8d'}", "addMoney");
                    } else {
                        this.showHurt();
                        _aElemToDelete.push(_aElemsInBoard[iRow][iCol]);
                        this.gameOver();
                    }
                } else {

                    this.showHurt();

                    _aElemToDelete.push(_aElemsInBoard[iRow][iCol]);

                    this.gameOver();
                }
                return;
            }
            this.showHurt();
            _aElemToDelete.push(_aElemsInBoard[iRow][iCol]);
        }


        for (var i = 0; i < _aElemToDelete.length; i++) {
            _aElemToDelete[i].remove();
            _iElemInGrid--;
        }

        this.compactGridVertically();
        this.moveElemsDown();
    };

    this.addMoney = function (line) {
        var json = JSON.parse(line);
        if (json.noFill) {
            if (json.action !== 'start') {
                if (json.amount == undefined) {
                    if (json.shown === 'false') {
                        if (json.canceled === 'false') {
                            if (confirm('Error occurred, try again ?')) {
                                if ('AppsgeyserJSInterface' in window) {
                                    AppsgeyserJSInterface.showAdColonyVideoAd("{'app_id':'app1d4ca5f8f68d475f95', 'client_options': 'version:1.0,store:google', 'zone_id': 'vz62f52d5f2d7b4c9b8d'}", "addMoney");
                                }
                            } else {
                                appsgeyser.data.setItem('lives', _iLives);
                                el.gameOver();
                            }
                        } else {
                            el.gameOver();
                        }
                    }
                }
            }
        } else {
            if (json.amount !== undefined) {
                _iLives = 3;
                el.refresh(_iLives);
            }
        }
    };

    this.elemOver = function (iRow, iCol, iType) {
        if (_iState !== GAME_STATE_NO_MOVE) {
            return;
        }

        _aElemToHighlight = new Array();
        this._findSimilarAdjacent(iRow, iCol, 0, iType, _aElemToHighlight);

        for (var i = 0; i < _aElemToHighlight.length; i++) {
            _aElemToHighlight[i].highlight(true);
        }
    };

    this.elemOut = function () {
        if (_aElemToHighlight === undefined) {
            return;
        }
        for (var i = 0; i < _aElemToHighlight.length; i++) {
            _aElemToHighlight[i].highlight(false);
        }
    };

    this.findElemInArray = function (oElem, aArrayToCheck) {
        for (var i = 0; i < aArrayToCheck.length; i++) {
            if (aArrayToCheck[i] === oElem) {
                return true;
            }
        }

        return false;
    };

    this.compactGridVertically = function () {
        var aColsChecked = new Array();
        _aElemToMove = new Array();
        _aCurMovingElems = new Array();
        for (var i = (BOARD_ROWS - 1); i > -1; i--) {
            for (var j = 0; j < BOARD_COLS; j++) {

                if (_aElemsInBoard[i][j].getType() === -1 && this.findElemInArray(j, aColsChecked) === false) {
                    this.findElemsToMoveVertically(i, j);
                    aColsChecked.push(j);
                }
            }
        }
    };

    this.findElemsToMoveVertically = function (iRow, iCol) {
        var iCurRow = iRow;
        var iDist = 0;

        while (iCurRow > -1) {
            if (_aElemsInBoard[iCurRow][iCol].getType() !== -1) {
                _aElemToMove.push({
                    row: iCurRow,
                    col: iCol,
                    type: _aElemsInBoard[iCurRow][iCol].getType(),
                    dist: iDist
                });

            } else {
                iDist += ELEM_SIZE;
            }
            iCurRow--;
        }
    };

    this.moveElemsDown = function () {
        if (_aElemToMove.length > 0) {
            _iState = GAME_STATE_MOVE_VERTICAL;
        } else {
            _iState = GAME_STATE_NO_MOVE;
            _aCurMovingElems = new Array();
            this.compactGridHorizontally();
            this.moveElemRight();
            return;
        }

        _iNumTweensActive = _aElemToMove.length;
        for (var i = 0; i < _aElemToMove.length; i++) {
            var iX = BOARD_OFFSET_X + (_aElemToMove[i].col * ELEM_SIZE);
            var iY = BOARD_OFFSET_Y + (_aElemToMove[i].row * ELEM_SIZE);
            var oMovingElem = new CMovingElem(iX, iY, _aElemToMove[i].type, _oElemSpriteSheet);
            _aCurMovingElems.push(oMovingElem);

            var iNewY = iY + _aElemToMove[i].dist;

            //UPDATE LOGIC GRID
            _aElemsInBoard[_aElemToMove[i].row][_aElemToMove[i].col].hide();
            var iNewRow = Math.floor((iNewY - BOARD_OFFSET_Y) / ELEM_SIZE);
            var iNewCol = _aElemToMove[i].col;

            oMovingElem.moveDown(iNewY, iNewRow, iNewCol);
        }
    };

    this.compactGridHorizontally = function () {
        _aElemToMove = new Array();
        for (var j = (BOARD_COLS - 1); j > -1; j--) {
            var bColEmpty = true;
            for (var i = 0; i < BOARD_ROWS; i++) {

                if (_aElemsInBoard[i][j].getType() !== -1) {
                    bColEmpty = false;
                    break;
                }
            }

            if (bColEmpty) {

                _iState = GAME_STATE_MOVE_HORIZONTAL;
                this.findElemsToMoveHorizontally(j);
            }
        }
    };

    this.findElemsToMoveHorizontally = function (iCol) {
        for (var i = 0; i < BOARD_ROWS; i++) {
            for (var j = 0; j < iCol; j++) {
                if (_aElemsInBoard[i][j].getType() !== -1) {
                    //CHECK IF THIS ELEM HAS ALREADY ADDED IN LIST TO MOVE. IF SO ADD SOME DISTANCE
                    var iCheck = this.checkElemToMove(i, j);
                    if (iCheck > -1) {
                        _aElemToMove[iCheck].dist += ELEM_SIZE;
                    } else {
                        _aElemToMove.push({row: i, col: j, type: _aElemsInBoard[i][j].getType(), dist: ELEM_SIZE});
                    }


                }
            }
        }
    };

    this.checkElemToMove = function (iRow, iCol) {
        for (var i = 0; i < _aElemToMove.length; i++) {
            if (_aElemToMove[i].row === iRow && _aElemToMove[i].col === iCol) {
                return i;
            }
        }

        return -1;
    };

    this.moveElemRight = function () {
        if (_aElemToMove.length === 0) {
            _iState = GAME_STATE_NO_MOVE;
            _aCurMovingElems = new Array();

            if (_iElemInGrid === 0) {
                this.showNextLevel();
            }
            return;
        }

        _iNumTweensActive = _aElemToMove.length;

        for (var i = 0; i < _aElemToMove.length; i++) {
            var iX = BOARD_OFFSET_X + (_aElemToMove[i].col * ELEM_SIZE);
            var iY = BOARD_OFFSET_Y + (_aElemToMove[i].row * ELEM_SIZE);
            var oMovingElem = new CMovingElem(iX, iY, _aElemToMove[i].type, _oElemSpriteSheet);
            _aCurMovingElems.push(oMovingElem);

            var iNewX = iX + _aElemToMove[i].dist;

            //UPDATE LOGIC GRID
            _aElemsInBoard[_aElemToMove[i].row][_aElemToMove[i].col].hide();
            var iNewRow = _aElemToMove[i].row;
            var iNewCol = Math.floor((iNewX - BOARD_OFFSET_X) / ELEM_SIZE);

            oMovingElem.moveSide(iNewX, iNewRow, iNewCol);
        }
    };

    this.updateElemValue = function (iRow, iCol, iType, oMovingSprite) {
        _aElemsInBoard[iRow][iCol].updateType(iType);

        s_oStage.removeChild(oMovingSprite);

        _iNumTweensActive--;
        if (_iNumTweensActive === 0) {
            _aCurMovingElems = new Array();
            if (_iState === GAME_STATE_MOVE_VERTICAL) {
                _iState = GAME_STATE_NO_MOVE;
                this.compactGridHorizontally();
                this.moveElemRight();
            } else if (_iState === GAME_STATE_MOVE_HORIZONTAL) {
                _iState = GAME_STATE_NO_MOVE;

                if (_iElemInGrid === 0) {
                    this.showNextLevel();
                }
            }
        }
    };

    this.gameOver = function () {
        _oEndPanel = CEndPanel(s_oSpriteLibrary.getSprite('msg_box'));
        _oEndPanel.show(_iScore);
    };

    this.showNextLevel = function () {
        $(s_oMain).trigger("end_level", _iCurLevel);
        _iCurLevel++;
        _oNextLevel = new CNextLevelPanel(s_oSpriteLibrary.getSprite("msg_box"));
        _oNextLevel.show(_iCurLevel);
    };

    this.showHurt = function () {
        playSound("life_lost", 1, 0);
        createjs.Tween.get(_oHurtFade).to({alpha: 0.5}, 300).call(function () {
            this.alpha = 0
        });
    };

    this.onExit = function () {
        _oInterface.unload();

        this.unload();

        $(s_oMain).trigger("end_level", _iCurLevel);
        $(s_oMain).trigger("end_session");
        $(s_oMain).trigger("share_event", _iScore);
        s_oMain.gotoMenu();
    };

    this.update = function () {
        for (var i = 0; i < _aCurMovingElems.length; i++) {
            _aCurMovingElems[i].update(SPEED_MOVE);
        }
    };

    s_oGame = this;

    BOARD_COLS = oInfo.col;
    BOARD_ROWS = oInfo.row;
    SCORE_PER_ELEM = oInfo.score_per_elem;
    SPEED_MOVE = oInfo.speed_move;
    NUM_LIVES = oInfo.num_lives;
    this._init();
}

var s_oGame;